; Game object handling


.p816   ; 65816 processor
.a16
.i16
.smart

.include "../../inc/engine/constants.inc"
.include "../../inc/engine/crashhandler.inc"

.import scratch, AddSprite
.export CreateObjectPool, SetObjectPools, CreateObject, DeleteObject, ResetAllObjects, UpdateObjects, MoveObject, Jmp_AnimateThenDrawThenNext, AnimateWithCallback, SetAnimation, ObjectWorldToTile, objPoolStart, objPoolEnd

_POOL_BEGIN = $00
_POOL_END = $02

.macro Macro_MapObjVar LONAME, MIDNAME, HINAME
    LONAME: .res 1
    .export LONAME
    MIDNAME: .res 1
    .export MIDNAME
    HINAME: .res OBJ_VAR_SIZE - 2
    .export HINAME
.endmacro

.macro Macro_MapObjVarWord NAME
    NAME: .res OBJ_VAR_SIZE
    .export NAME
.endmacro

.macro Macro_MapObjVarByteWord BYTENAME, WORDNAME
    BYTENAME: .res 1
    .export BYTENAME
    WORDNAME: .res OBJ_VAR_SIZE - 1
    .export WORDNAME
.endmacro

.segment "LORAM"

Macro_MapObjVar objPosSubX, objPosX, objPosHiX
Macro_MapObjVar objPosSubY, objPosY, objPosHiY
Macro_MapObjVarByteWord objCustomByteA, objVelX
Macro_MapObjVarByteWord objCustomByteB, objVelY
Macro_MapObjVarByteWord objCustomByteC, objSizeX
Macro_MapObjVarByteWord objCustomByteD, objSizeY
Macro_MapObjVarByteWord objTimer, objAnimDef
Macro_MapObjVarByteWord objCustomByteE, objAttribute
Macro_MapObjVarWord objCustomA
Macro_MapObjVarWord objCustomB
Macro_MapObjVarWord objCustomC
Macro_MapObjVarWord objCustomD

Macro_MapObjVarWord objThinker
_objThinkSentinel: .res 2

objPoolHead: .res 16
objPoolTail: .res 16
objPoolStart: .res 16
objPoolEnd: .res 16
objPool: .res OBJ_MAX_COUNT * 2

_callbackScratch: .res 2

.segment "CODE0HI"

_DefaultObjectPool:
    .word $0000, OBJ_MAX_COUNT, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff, $ffff

_EmptyThinker:
    inx
    inx
    inx
    jmp (objThinker, x)

Jmp_AnimateThenDrawThenNext:
    sep #$20
    dec objTimer, x
    rep #$20
    beq _NextAnimation
    ldy objAnimDef, x
    
_DoDraw:
    lda a:ANIMDEF_DATA, y
    tay
    jsr AddSprite
    
    inx
    inx
    inx
    jmp (objThinker, x)

_NextAnimation:
    ldy objAnimDef, x
    lda a:ANIMDEF_NEXT, y
    sta objAnimDef, x
    tay
    sep #$20
    lda a:ANIMDEF_DELAY, y
    sta objTimer, x
    rep #$20
    
    bra _DoDraw

AnimateWithCallback:
    sep #$20
    dec objTimer, x
    rep #$20
    beq _NextAnimationCallback
    rts

_NextAnimationCallback:
    ldy objAnimDef, x
    lda a:ANIMDEF_NEXT, y
    sta objAnimDef, x
    tcd
    sep #$20
    lda z:ANIMDEF_DELAY
    sta objTimer, x
    rep #$20
    lda z:ANIMDEF_DATA
    bne _DoCallback
    
    rts

_DoCallback:
    sta _callbackScratch
    jmp (_callbackScratch)

SetAnimation:
    tya
    sta objAnimDef, x
    lda a:ANIMDEF_DELAY, y
    sep #$20
    sta objTimer, x
    rep #$20
    
    rts

; ----
; Expects anything but chops XY, clobbers A, X
; ----
CreateObjectPool:
    php
    rep #$30
    
    lda #$0000
    ldx #$0000
_CreatePoolLoop:
    sta objPool, x
    clc
    adc #$03
    inx
    inx
    cpx #OBJ_MAX_COUNT * 2
    bne _CreatePoolLoop
    
    ; Default object pool size
    
    ldx #.loword(_DefaultObjectPool)
    jsr SetObjectPools
    
    plp
    rts

; ----
; In: X - pointer to list of start/end values for each pool
; ----
; Expects anything, clobbers A, X, Y
; ----
SetObjectPools:
    php
    rep #$30
    
    ldy #$0000
    
_SetObjectPoolLoop:
    lda a:_POOL_BEGIN, x
    asl
    sta objPoolStart, y
    
    lda a:_POOL_END, x
    asl
    sta objPoolEnd, y
    dec a
    dec a
    sta objPoolHead, y
    sta objPoolTail, y
    
    inx
    inx
    inx
    inx
    iny
    iny
    
    cpy #16
    bcc _SetObjectPoolLoop
    
    plp
    rts

; ----
; In: Y - index of the pool to use (word sized)
; ----
; Expects AXY16, clobbers A, X
; ----
CreateObject:
    lda objPoolHead, y
    bmi _NoObjectsLeft
    clv ; We're placing the object, caller doesn't need to clv
    ; Next slot, wrap if necessary
    inc a
    inc a
    cmp objPoolEnd, y
    bpl _WrapObjectPool
_WrapObjectPoolDone:
    cmp objPoolTail, y
    beq _FilledObjectPool
    
    sta objPoolHead, y
    
_FilledObjectPoolDone:
    tax
    lda objPool, x
    tax
    
    rts
    
_FilledObjectPool:
    ; We mark the object pool as "full" by setting the sign bit.
    ; Otherwise there is no way to tell a difference between an
    ; empty and full pool, since both will have same head and
    ; tail indices.
    ora #%1000000000000000
    sta objPoolTail, y
    sta objPoolHead, y
    and #%0111111111111111
    bra _FilledObjectPoolDone

_NoObjectsLeft:
    sep #$40 ; Overflow flag indicates object was not created
    rts

_WrapObjectPool:
    lda objPoolStart, y
    bra _WrapObjectPoolDone

; ----
; In: X - object being released back into the pool
; In: Y - index of the pool to use (word sized)
; ----
; Expects AXY16, clobbers A, X, Y
; ----
DeleteObject:
    lda #.loword(_EmptyThinker)
    sta objThinker, x
    
    lda objPoolTail, y
    bmi _UnfilledObjectPool
_UnfilledObjectPoolDone:
    inc a
    inc a
    cmp objPoolEnd, y
    bpl _WrapObjectPoolTail
_WrapObjectPoolTailDone:
    sta objPoolTail, y
    
    tay
    txa
    sta objPool, y
    
    rts

_UnfilledObjectPool:
    ; See above, we clear the sign bit to indicate pool
    ; is no longer full.
    and #%0111111111111111
    sta objPoolHead, y
    bra _UnfilledObjectPoolDone

_WrapObjectPoolTail:
    lda objPoolStart, y
    bra _WrapObjectPoolTailDone

; ----
; Expects anything, clobbers A, X
; ----
ResetAllObjects:
    php
    rep #$30
    
    ldx #OBJ_VAR_SIZE - 3
_ResetAllObjectsLoop:
    lda #.loword(_EmptyThinker)
    sta objThinker, x
    dex
    dex
    dex
    bpl _ResetAllObjectsLoop
    plp
    rts    

; ----
; Expects anything, clobbers X
; ----
UpdateObjects:
    php
    rep #$30
    
    ; Function pointer after the last object jumps to termination of loop
    lda #.loword(_UpdateObjectsDone)
    sta _objThinkSentinel
    
    ldx #$00
_UpdateObjectsLoop:
    jsr (objThinker, x)
    inx
    inx
    inx
    bra _UpdateObjectsLoop

_UpdateObjectsDone:
    pla ; We jsr and then jmp here, so need to pop a word from the stack
    plp
    rts

; ----
; Expects AXY16, clobbers A
; ----
MoveObject:
    lda objVelX, x
    ; We need separate logic for moving in the positive and
    ; negative directions. This is so we know whether to inc
    ; or dec the high bit based on the carry bit status.
    bmi _MoveNegX
    
    ; Positive X
    clc
    adc objPosSubX, x
    sta objPosSubX, x
    bcs _IncHighX
    
    bra _MoveObjectY
    
_MoveNegX:
    
    clc
    adc objPosSubX, x
    sta objPosSubX, x
    bcc _DecHighX
    
    bra _MoveObjectY

_IncHighX:
    
    inc objPosHiX, x
    
    bra _MoveObjectY

_DecHighX:

    dec objPosHiX, x

_MoveObjectY:
    
    lda objVelY, x
    bmi _MoveNegY
    
    ; Positive Y
    clc
    adc objPosSubY, x
    sta objPosSubY, x
    bcs _IncHighY
    
    rts
    
_MoveNegY:
    
    clc
    adc objPosSubY, x
    sta objPosSubY, x
    bcc _DecHighY
    
    rts

_IncHighY:
    
    inc objPosHiY, x
    
    rts

_DecHighY:

    dec objPosHiY, x
    
    rts

; ----
; Expects AXY16, clobbers A
; ----
ObjectWorldToTile:
    lda objPosSubX, x
    lsr
    lsr
    lsr
    lsr
    and #%11111
    sta scratch+2
    
    lda objPosSubY, x
    asl
    and #%0000001111100000
    ora scratch+2
    
    rts
