local app = require "src.app"
local oops = require "src.utils.oops"

local DropTargetScreen = require "src.screens.droptargetscreen"
local EditorScreen = require "src.screens.editorscreen"

local PaletteScreen = require "src.screens.palettescreen"
local CharScreen = require "src.screens.charscreen"
local TilemapScreen = require "src.screens.tilemapscreen"

local MainScreen = DropTargetScreen:makeClass
{
    message = "Drag a file onto this window to edit it.\n\n\nSupported filetypes:",
    timer = 0
}

MainScreen.filetypeMapping =
{
    {
        extension = ".pal",
        description = "Palette",
        purpose = "View and edit raw SNES palette files",
        handler = function(self, file)
            app.setScreen(PaletteScreen:new(file))
        end
    },
    {
        extension = ".pic",
        description = "Char GFX",
        purpose = "View raw 4bpp SNES char data",
        handler = function(self, file)
            app.setScreen(CharScreen:new(file))
        end
    },
    {
        extension = ".pic2",
        description = "BG3 char GFX",
        purpose = "View raw 2bpp SNES char data",
        handler = function(self, file)
            app.setScreen(CharScreen:new(file))
        end
    },
    {
        extension = ".map",
        description = "Tilemap",
        purpose = "View and edit raw SNES tilemaps",
        handler = function(self, file)
            app.setScreen(TilemapScreen:new(file))
        end
    },
}

function MainScreen:badDrop(file)
    self.message = "Unknown file extension. Please try a different file."
end

function MainScreen:init(message)
    DropTargetScreen.init(self)
    
    if message then
        self.message = message
    end
    
    self.animating = true
end

function MainScreen:update(dt)
    -- Workaround for desktop effects switch giving blank window - force updates for a couple seconds
    if self.timer < 2 then
        self.timer = self.timer + dt
        self.animating = true
        self.dirty = true
    else
        self.animating = false
    end
end

function MainScreen:draw()
    love.graphics.setColor(255, 255, 255)
    love.graphics.setFont(app.res.uifont)
    
    local prompt = self.message .. "\n\n"
    
    for i, mapping in ipairs(self.filetypeMapping) do
        prompt = prompt .. "                [" .. mapping.extension .. "]: " .. mapping.description .. "\n"
                        .. "                                " .. mapping.purpose .. "\n"
    end
    
    love.graphics.print(prompt, 32, 32)
end

return MainScreen
