local app = require "src.app"
local extramath = require "src.utils.extramath"

local EditorScreen = require "src.screens.editorscreen"

local TilePickerScreen = EditorScreen:makeClass
{
    handlers =
    {
        keypressed = EditorScreen.handlers.keypressed,
        mousepressed = EditorScreen.handlers.mousepressed,
        mousereleased = EditorScreen.handlers.mousereleased,
        mousemoved = EditorScreen.handlers.mousemoved,
        wheelmoved = EditorScreen.handlers.wheelmoved,
    },
    rowSize = 16,
    clipboardId = "TilePicker"
}

TilePickerScreen.keyHandlers =
{
    escape = function()
        app.popScreen()
    end,
    left = function(self)
        self:setSelectedTile(self.pickerData.index - 1)
        self.needScroll = true
    end,
    right = function(self)
        self:setSelectedTile(self.pickerData.index + 1)
        self.needScroll = true
    end,
    up = function(self)
        self:setSelectedTile(self.pickerData.index - self.columnSize)
        self.needScroll = true
    end,
    down = function(self)
        self:setSelectedTile(self.pickerData.index + self.columnSize)
        self.needScroll = true
    end,
    ["`"] = function(self)
        self:setPalette(0)
    end,
    ["0"] = function(self)
        self:setPalette(0)
    end,
    ["1"] = function(self)
        self:setPalette(1)
    end,
    ["2"] = function(self)
        self:setPalette(2)
    end,
    ["3"] = function(self)
        self:setPalette(3)
    end,
    ["4"] = function(self)
        self:setPalette(4)
    end,
    ["5"] = function(self)
        self:setPalette(5)
    end,
    ["6"] = function(self)
        self:setPalette(6)
    end,
    ["7"] = function(self)
        self:setPalette(7)
    end,
}

function TilePickerScreen:mousePressedContent(x, y, button, w, h)
    if button == 1 then
        
    end
end

function TilePickerScreen:mouseMovedContent(x, y, button, w, h)
    if love.mouse.isDown(1) then
        
    end
end

function TilePickerScreen:setSelectedTile(index)
    self.pickerData.index = extramath.clampValue(0, #self.quads[0], index)
    self.dirty = true
end

function TilePickerScreen:setPalette(palette)
    self.pickerData.palette = palette % 8
    self.dirty = true
end

function TilePickerScreen:getTileScale(width)
    local scale = width / (self.texture:getWidth() / 8)
    
    if scale >= 1 then scale = math.floor(scale) end
    
    return scale
end

function TilePickerScreen:init(texture, quads, pickerData)
    EditorScreen.init(self)
    
    if not texture or not quads or not pickerData then error("Where's the texture, quads, and picker data??") end
    
    self.texture = texture
    self.quads = quads
    self.pickerData = pickerData
end

function TilePickerScreen:drawContent(width, height)
    local scale = self:getTileScale(width)
    local textureHeight = self.texture:getHeight() * scale
    local scrollSize = math.max(0, textureHeight - height)
    
    self:updateScrollInfo(0, scrollSize, 32, scrollSize > 0 and height / textureHeight or 1)
    
    local index = self.pickerData.index
    local palette = self.pickerData.palette % 8
    
    love.graphics.push()
    love.graphics.translate(0, -self.contentScroll)
    
    love.graphics.draw(self.texture, self.quads[palette], 0, 0, 0, scale, scale)
    
    -- Draw selection rectangle
    local tileX = (index % self.columnSize) * 8 * scale
    local tileY = math.floor(index / self.columnSize) * 8 * scale
    
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.setLineWidth(2)
    love.graphics.setLineStyle("smooth")
    love.graphics.setScissor()
    love.graphics.rectangle("line", tileX, tileY, scale * 8, scale * 8)
    
    love.graphics.pop()
end

function TilePickerScreen:drawSidebar(width, height)
    
end

return TilePickerScreen
